import type { DataType } from '../types.js';
import type { parseOptions } from './types.js';
/**
 * Inferer: safely infering data for managing and transforming data
 */
export declare class Inferer {
    _data: DataType;
    constructor(data: DataType);
    /**
     * create error object
     *
     * @private
     * @param {...string[]} tags
     * @return {Error}
     * @memberof Inferer
     */
    private _createError;
    /**
     * fallback handler for each infer type function
     *
     * @private
     * @template T
     * @param {string} path
     * @param {string} type
     * @param {DataType} value
     * @param {parseOptions<T>} options
     * @return {T}
     * @memberof Inferer
     */
    private _fallbackHandler;
    /**
     * Get raw value based on object path
     */
    get(path?: string): DataType;
    /**
     * check given path if value exist or not.
     *
     * @param {string} path
     * @param {(data: DataType) => boolean} [validator=(v) => v !== undefined] default to `(v) => v !== undefined`
     * @return {boolean}
     */
    has(path: string, validator?: (data: DataType) => boolean): boolean;
    /**
     * Infer object path in runtime either value is string or throw an error
     *
     * @param {string} [path='']
     * @param {parseOptions<string>} [options={}]
     * @return {string}
     * @memberof Inferer
     */
    string(path?: string, options?: parseOptions<string>): string;
    /**
     * Infer object path in runtime either value is number (int|float) or throw an error
     *
     * @param {string} [path='']
     * @param {(parseOptions<number> & { float?: boolean })} [options={}]
     * @return {number}
     * @memberof Inferer
     */
    number(path?: string, options?: parseOptions<number> & {
        float?: boolean;
    }): number;
    /**
     * Infer object path in runtime either value is boolean or throw an error
     *
     * @param {string} [path='']
     * @param {parseOptions<boolean>} [options={}]
     * @return {boolean}
     * @memberof Inferer
     */
    boolean(path?: string, options?: parseOptions<boolean>): boolean;
    /**
     * Infer object path in runtime either value is string | number or throw an error
     *
     * @param {string} [path='']
     * @param {parseOptions<string | number>} [options={}]
     * @return {string | number}
     * @memberof Inferer
     */
    stringOrNumber(path?: string, options?: parseOptions<string | number>): string | number;
    /** alias for stringOrNumber */
    snum(path?: string, options?: parseOptions<string | number>): string | number;
    /**
     * Infer object path in runtime is array then return as InfererArray object.
     *
     * @param {boolean} [defaultEmpty=false] when true return will fallback to empty array. On false will throw error. default: false.
     */
    array(path?: string, defaultEmpty?: boolean): InfererArray;
    /**
     * Infer object path when path has value it will parse or transform with given function or throw an error when value is null or undefined.
     */
    parse<T = DataType>(path: string, fn: (data: unknown) => T, options?: parseOptions<T>): T;
    /**
     * Infer object path and fallback to given preset value and type if type not match
     */
    maybe<T>(path: string, preset: T): {
        /** check if given path is a string or return preset value */
        string(options?: parseOptions<string>): string | T;
        /** check if given path is a number or return preset value */
        number(options?: parseOptions<number> & {
            float?: boolean;
        }): number | T;
        /** check if given path is a boolean or return preset value */
        boolean(options?: parseOptions<boolean>): boolean | T;
        /** check if given path is a string, number or return preset value */
        stringOrNumber(options?: parseOptions<string | number>): string | number | T;
        /** check if given path has value then parse using given function or return preset value */
        parse<V>(fn: (data: unknown) => V, options?: parseOptions<V>): T | V;
        /** check if given path is array then return InfererArray or return preset value */
        array(): InfererArray | T;
    };
    /**
     * alias or shortcut for maybe with preset value null
     */
    nullable(path?: string): {
        /** check if given path is a string or return preset value */
        string(options?: parseOptions<string>): string | null;
        /** check if given path is a number or return preset value */
        number(options?: parseOptions<number> & {
            float?: boolean | undefined;
        }): number | null;
        /** check if given path is a boolean or return preset value */
        boolean(options?: parseOptions<boolean>): boolean | null;
        /** check if given path is a string, number or return preset value */
        stringOrNumber(options?: parseOptions<string | number>): string | number | null;
        /** check if given path has value then parse using given function or return preset value */
        parse<V>(fn: (data: unknown) => V, options?: parseOptions<V>): V | null;
        /** check if given path is array then return InfererArray or return preset value */
        array(): InfererArray | null;
    };
}
/**
 * Extended array object with typesafe infering and transform capability
 *
 * @export
 * @class InfererArray
 * @extends {Array}
 */
export declare class InfererArray extends Array {
    constructor(collection: DataType[]);
    /**
     * loop each item and parse or transform item using given function.
     *
     * @param {(row: Inferer, index: number, arr: InfererArray) => T} fn
     * @return {T[]} new transformed array
     */
    inferEach<T = DataType>(fn: (row: Inferer, index: number, arr: InfererArray) => T): T[];
    /**
     * extended Array.at that return inferer object.
     *
     * @param {number} index The zero-based index of the desired code unit. A negative index will count back from the last item.
     */
    inferAt(index: number): Inferer;
}
