import { PermitNode, SchemaNode } from '../node.js';
import { keySafe } from '../utils.js';
// eslint-disable-next-line @typescript-eslint/no-explicit-any
export class Permit {
    /** Access Schema */
    on;
    _nodes = new Map();
    schema;
    scopes;
    constructor(schema, scopes = []) {
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        const travel = (branch) => {
            const entries = [];
            for (const key in branch) {
                if (Object.prototype.hasOwnProperty.call(branch, key)) {
                    const value = branch[key];
                    if (typeof key !== 'string')
                        break;
                    if (value instanceof SchemaNode) {
                        // value is schema node instance
                        // create permit node directly to value
                        const node = new PermitNode(value, scopes.includes(value.$scope));
                        this._nodes.set(node.$scope, node);
                        entries.push([key, node]);
                    }
                    else if (Object.hasOwn(value, '$scope') && Object.hasOwn(value, '$description')) {
                        // value is native object that has $scope and $description
                        // create schema node first then do the rest
                        const schemanode = new SchemaNode(value.$scope, value.$description);
                        const node = new PermitNode(schemanode, scopes.includes(schemanode.$scope));
                        this._nodes.set(node.$scope, node);
                        entries.push([key, node]);
                    }
                    else if (key.startsWith('$') || key.startsWith('_')) {
                        // passthrough utility
                        entries.push([key, value]);
                    }
                    else {
                        // value is on branch not node
                        const node = travel(value);
                        const $branch = new PermitBranch(node);
                        entries.push([key, Object.assign($branch, node)]);
                    }
                }
            }
            return Object.fromEntries(entries);
        };
        this.scopes = scopes;
        this.schema = schema;
        const root = travel(this.schema);
        const tree = new PermitBranch(root);
        this.on = Object.assign(tree, root);
    }
    static getNodeScope(scope) {
        return typeof scope === 'string' ? scope : scope.$scope;
    }
    /* -------------------------------------------------------------------------- */
    /** get nodes that has active access or granted */
    get granted() {
        return this.on.$granted;
    }
    /** get nodes that is inactive or denied access */
    get denied() {
        return this.on.$denied;
    }
    /** get list of permit node */
    get nodes() {
        return this.on.$nodes;
    }
    /** reset scope to default state */
    reset() {
        this.on.$each((node) => {
            if (this.scopes.includes(node.$scope)) {
                node.$able = true;
            }
            else {
                node.$able = false;
            }
        });
    }
    /** grant access to scope */
    grant(...scopes) {
        for (let i = 0; i < scopes.length; i++) {
            const scope = Permit.getNodeScope(scopes[i]);
            const node = this._nodes.get(scope);
            if (node)
                node.$able = true;
        }
    }
    /** deny access to scope */
    deny(...scopes) {
        for (let i = 0; i < scopes.length; i++) {
            const scope = Permit.getNodeScope(scopes[i]);
            const node = this._nodes.get(scope);
            if (node)
                node.$able = false;
        }
    }
    ableTo(scope) {
        const $scope = Permit.getNodeScope(scope);
        const node = this._nodes.get($scope);
        if (node)
            return node.$able;
        return false;
    }
    get(scope) {
        const $scope = Permit.getNodeScope(scope);
        return this._nodes.get($scope);
    }
    has(scope) {
        const $scope = Permit.getNodeScope(scope);
        const node = this._nodes.get($scope);
        return node !== undefined;
    }
    child(scope) {
        const child = [];
        this.on.$each((node) => node.$scope.startsWith(scope) && child.push(node));
        return child;
    }
    /** loop through each node */
    $each(fn) {
        this.on.$each(fn);
    }
}
// eslint-disable-next-line @typescript-eslint/no-explicit-any
export class PermitBranch {
    $tree;
    constructor(nodes) {
        this.$tree = nodes;
    }
    $each(fn) {
        for (const key in this.$tree) {
            if (keySafe(key) && Object.prototype.hasOwnProperty.call(this.$tree, key)) {
                // eslint-disable-next-line @typescript-eslint/no-explicit-any
                const node = this.$tree[key];
                if (node instanceof PermitNode) {
                    fn(node);
                }
                else if (node instanceof PermitBranch) {
                    node.$each(fn);
                }
            }
        }
    }
    get $able() {
        let valid = true;
        this.$each((node) => (valid = valid && node.$able));
        return valid;
    }
    get $doable() {
        let valid = false;
        this.$each((node) => (valid = valid || node.$able));
        return valid;
    }
    get $disable() {
        return this.$doable === false;
    }
    get $nodes() {
        const nodes = [];
        this.$each((node) => nodes.push(node));
        return nodes;
    }
    get $granted() {
        const nodes = [];
        this.$each((node) => {
            if (node.$able)
                nodes.push(node);
        });
        return nodes;
    }
    get $denied() {
        const nodes = [];
        this.$each((node) => {
            if (!node.$able)
                nodes.push(node);
        });
        return nodes;
    }
}
