/* eslint-disable @typescript-eslint/no-explicit-any */
import { SchemaBranch } from './branch/schema.js';
import { SchemaNode } from './node.js';
/**
 * traverse given schema tree by recursive loop
 * then call hook onLeaf on every leaf node
 * and onBranch on every branch node.
 */
export function traverseTree(node, parent = '', spacer = '.', onLeaf = () => undefined, onBranch = () => undefined) {
    for (const key in node) {
        if (Object.prototype.hasOwnProperty.call(node, key)) {
            const scope = parent !== '' ? [parent, key].join(spacer) : key;
            const current = node[key];
            if (typeof current === 'string') {
                onLeaf(current, scope, parent);
            }
            else {
                onBranch(current, scope, parent);
                traverseTree(current, scope, spacer, onLeaf, onBranch);
            }
        }
    }
}
/**
 * transform given schema tree by recursive loop
 * then on every leaf transform leaf node using onLeaf hook.
 */
export function transformTree(nodes, parent, spacer = '.', onLeaf) {
    const traverse = (node, path) => {
        const scope = parent !== '' ? [parent, path].join(spacer) : path;
        if (typeof node === 'string') {
            const value = onLeaf(node, scope, path);
            return [path, value];
        }
        else {
            return [path, transformTree(node, scope, spacer, onLeaf)];
        }
    };
    return Object.fromEntries(Object.entries(nodes).map(([key, value]) => traverse(value, key)));
}
export const makeScope = (spacer, paths) => paths.filter((c) => c && c !== '').join(spacer);
export const keySafe = (key) => typeof key === 'string' && !key.startsWith('$') && !key.startsWith('_');
export const loopSchema = (schema, onNode, onBranch) => {
    for (const key in schema) {
        if (keySafe(key) && Object.prototype.hasOwnProperty.call(schema, key)) {
            const node = schema[key];
            if (node instanceof SchemaNode) {
                onNode(node, key);
            }
            else if (node instanceof SchemaBranch) {
                onBranch(node, key);
            }
        }
    }
};
